#!/usr/bin/env bash

BLCKLST_NET="10.1.0.0/16 10.152.183.0/24"
IP="$(ip route get 1 | awk '{print $(NF-2);exit}')"

if [[ -z ${IP} ]]; then
  echo "\e[93mThe installation requires a default gateway\e[0m"
  echo "\e[93mInstall the default gateway and retry the installation\e[0m"
  return 1;
fi

function calc_subnet() {
    local ip ip_a mask netmask sub sub_ip rval start end
    local readonly BITMASK=0xFFFFFFFF

    IFS=/ read sub mask <<< "${1}"
    IFS=. read -a sub_ip <<< "${sub}"
    IFS=. read -a ip_a <<< "${2}"

    netmask=$(($BITMASK<<$((32-$mask)) & $BITMASK))

    start=0
    for o in "${sub_ip[@]}"
    do
        start=$(($start<<8 | $o))
    done

    start=$(($start & $netmask))
    end=$(($start | ~$netmask & $BITMASK))

    ip=0
    for o in "${ip_a[@]}"
    do
        ip=$(($ip<<8 | $o))
    done

    (( $ip >= $start )) && (( $ip <= $end )) && rval=1 || rval=0
    echo "${rval}"
}

function check_ip() {
  CNI_NET="$(cat /var/snap/microk8s/current/args/kube-proxy | grep cluster-cidr | cut -f2 -d"=")"
  SVC_NET="$(cat /var/snap/microk8s/current/args/kube-apiserver | grep service-cluster-ip-range | cut -f2 -d"=")"
  CNI_NET_NEW="10.20.0.0/16"
  SVC_NET_NEW="10.172.183.0/24"

  for SUBNET in $BLCKLST_NET; do
    (( $(calc_subnet $SUBNET $IP) )) && ERR_NET=${SUBNET} && ERR_MSG_IP="${IP} is in ${SUBNET}"
  done

  if [[ ${ERR_NET} == ${SVC_NET} || ${ERR_NET} == ${CNI_NET} ]]; then
    echo -e "\e[93mLocal IP address intersects with Kubernetes network ($ERR_MSG_IP)"
    if [[ ${ERR_NET} == ${CNI_NET} ]]; then
      echo -e "New CIDR values for CNI = $CNI_NET_NEW"
    elif [[ ${ERR_NET} == ${SVC_NET} ]]; then
      echo -e "Terminated, need to change IP address outside the following ranges ${BLCKLST_NET}\e[0m"
      return 1
    fi
    echo -e "Changing the pods CIDR in a MicroK8s cluster?(y/N)\e[0m"
    read -r yn
    case $yn in
        [Yy]* ) replace_ip;;
        [Nn]* ) echo -e "terminated\n\nChange CIDR and try again"; return 1;;
        * ) echo -e "terminated\n\nChange CIDR and try again"; return 1;;
    esac
  fi
}

function replace_ip() {
  if [[ ${ERR_NET} == ${CNI_NET} ]]; then
    microk8s kubectl delete -f /var/snap/microk8s/current/args/cni-network/cni.yaml
    sed -i'' "s|${CNI_NET}|${CNI_NET_NEW}|g" /var/snap/microk8s/current/args/kube-proxy
    microk8s stop && microk8s start
    microk8s status --wait-ready > /dev/null
    sed -i'' "s|${CNI_NET}|${CNI_NET_NEW}|g" /var/snap/microk8s/current/args/cni-network/cni.yaml
    microk8s kubectl apply -f /var/snap/microk8s/current/args/cni-network/cni.yaml
    microk8s status --wait-ready > /dev/null
  fi
  check_ip
}

check_ip
