#!/usr/bin/env bash

## проерки host os

HAS_ERRORS=false

if ! type "curl" > /dev/null && ! type "wget" > /dev/null; then
  echo -e "\e[91mEither curl or wget is required\e[39m"
  HAS_ERRORS=true
fi

if ! type "snap" > /dev/null; then
  echo -e "\e[91mSnapd required\e[39m"
  HAS_ERRORS=true
fi

if [[ $HOSTNAME =~ [[:upper:]] ]]; then
  echo -e "\e[91mWarning! Hostname: ${HOSTNAME} - contains capital letters.\e[39m"
  echo "For successful installation set hostname without capital letters."
  echo "Run 'hostnamectl set-hostname newhostname' to set new hostname."
  echo "Then reboot and start this installation again."
  HAS_ERRORS=true
fi


ARCH=$(uname -m)
case $ARCH in
  armv5*) ARCH="armv5";;
  armv6*) ARCH="armv6";;
  armv7*) ARCH="arm";;
  aarch64) ARCH="arm64";;
  x86) ARCH="386";;
  x86_64) ARCH="amd64";;
  i686) ARCH="386";;
  i386) ARCH="386";;
esac

# initOS discovers the operating system for this system.
OS=$(uname | tr '[:upper:]' '[:lower:]')

case "$OS" in
  # Minimalist GNU for Windows
  mingw*) OS='windows';;
esac

supported="linux-amd64"
if ! echo "${supported}" | grep -q "${OS}-${ARCH}"; then
  echo -e "\e[91mNo prebuilt binary for ${OS}-${ARCH}.\e[39m"
  HAS_ERRORS=true
fi

if [ "${HAS_ERRORS}" == "true" ]; then
  return 1;
fi
