#!/usr/bin/env bash

#HTTPS_PROXY="https://proxy_user:proxy_pass@host:port/"
#HTTP_PROXY="http://proxy_user:proxy_pass@host:port/"
#NO_PROXY="localhost,127.0.0.0/8,10.0.0.0/8,169.254.0.0/16,192.168.0.0/16,100.64.0.0/10,172.16.0.0/12"
#https_proxy="https://proxy_user:proxy_pass@host:port/"
#http_proxy="http://proxy_user:proxy_pass@host:port/"
#no_proxy="localhost,127.0.0.0/8,10.0.0.0/8,169.254.0.0/16,192.168.0.0/16,100.64.0.0/10,172.16.0.0/12"

function askProxy() {
  echo -e "\e[93mDo you want to specify your proxy settings (/etc/environment)?(y/N)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) enterProxy; return 0;;
      [Nn]* ) if [[ ${ERR_PROXY} == true ]]; then echo "terminated"; return 1; else echo "skipping"; return 0;fi;;
      * ) if [[ ${ERR_PROXY} == true ]]; then echo "terminated"; return 1; else echo "skipping"; return 0;fi;;
  esac
}

function enterProxy() {
  echo "[ Setup proxy ]"
  echo "Specify your host, port and password."
  echo -n "Host(ex:192.168.1.1 or example.org): "
  read PROXY_HOST
  while [ -z "${PROXY_HOST}" ]; do
    echo -e "\e[31mERR\e[0m: Host value must not be empty"
    echo -n "Host(ex:192.168.1.1 or example.org): "
    read PROXY_HOST
  done
  echo -n "Port: "
  read PROXY_PORT
  while [ -z "${PROXY_PORT}" ]; do
    echo -e "\e[31mERR\e[0m: Port value must not be empty"
    echo -n "Port: "
    read PROXY_PORT
  done
  echo -n "Username: "
  read PROXY_USER
  if [[ $PROXY_USER != "" ]]; then
    echo -n "Password: "
    read -es PROXY_PASSWORD
    local PROXY_CREDS="$PROXY_USER:$PROXY_PASSWORD@"
  fi
  host=$(hostname)
  PROXY_STRING="http://$PROXY_CREDS$PROXY_HOST:$PROXY_PORT/"
  NOPROXY_STRING="localhost,127.0.0.0/8,10.0.0.0/8,169.254.0.0/16,192.168.0.0/16,100.64.0.0/10,172.16.0.0/12,${host}"
  setProxy
  viewProxy
  applyProxy
}

function setProxy() {
  unsetProxy
  export HTTPS_PROXY=$PROXY_STRING
  export HTTP_PROXY=$PROXY_STRING
  export NO_PROXY=$NOPROXY_STRING
  export https_proxy=$PROXY_STRING
  export http_proxy=$PROXY_STRING
  export no_proxy=$NOPROXY_STRING
sudo tee -a /etc/environment <<EOF >/dev/null
HTTPS_PROXY=$PROXY_STRING
HTTP_PROXY=$PROXY_STRING
NO_PROXY=$NOPROXY_STRING
https_proxy=$PROXY_STRING
http_proxy=$PROXY_STRING
no_proxy=$NOPROXY_STRING
EOF
}

function applyProxy() {
  echo -e "\e[93mTo apply the settings, you must log out and login again."
  echo -e "\e[93mThen run the installation again."
  echo -e "\e[93mPress any key to exit.\e[0m"
  read -r yn
  case $yn in
      * ) return 1;;
  esac
}

function unsetProxy() {
#  sed -i '/_proxy=/Id' /etc/environment
   sed -i '/HTTPS_PROXY/Id' /etc/environment
   sed -i '/HTTP_PROXY/Id' /etc/environment
   sed -i '/NO_PROXY/Id' /etc/environment
   sed -i '/https_proxy/Id' /etc/environment
   sed -i '/http_proxy/Id' /etc/environment
   sed -i '/no_proxy/Id' /etc/environment
  unset HTTPS_PROXY
  unset HTTP_PROXY
  unset NO_PROXY
  unset https_proxy
  unset http_proxy
  unset no_proxy
  echo -e "\e[93mProxy settings removed\e[0m"
}

function viewProxy() {
  echo -e "\nYour proxy settings:"
  echo "  HTTPS_PROXY=${HTTPS_PROXY}"
  echo "  HTTP_PROXY=${HTTP_PROXY}"
  echo "  NO_PROXY=${NO_PROXY}"
  echo "  https_proxy=${https_proxy}"
  echo "  http_proxy=${http_proxy}"
  echo "  no_proxy=${no_proxy}"
  echo -e "\e[93mContinue with current settings?(Y/n) Remove proxy settings(d)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) echo "Settings accepted"; return 0;;
      [Nn]* ) enterProxy;;
      [Dd]* ) unsetProxy; applyProxy;;
      * ) echo "Settings accepted"; return 0;;
  esac
}

function checkProxy() {
  if [[ -z ${HTTPS_PROXY} ]] && [[ -z "${HTTP_PROXY}" ]] && [[ -z "${NO_PROXY}" ]] \
  && [[ -z "${https_proxy}" ]] && [[ -z "${http_proxy}" ]] && [[ -z "${no_proxy}" ]]; then
    ERR_PROXY=false
    askProxy
  elif [[ -n "${HTTPS_PROXY}" ]] && [[ -n "${HTTP_PROXY}" ]] && [[ -n "${NO_PROXY}" ]] \
  && [[ -n "${https_proxy}" ]] && [[ -n "${http_proxy}" ]] && [[ -n "${no_proxy}" ]]; then
    ERR_PROXY=false
    viewProxy
  elif [[ -n "${HTTPS_PROXY+set}" ]] || [[ -n "${HTTP_PROXY+set}" ]] || [[ -n "${NO_PROXY+set}" ]] \
  || [[ -n "${https_proxy+set}" ]] || [[ -n "${http_proxy+set}" ]] || [[ -n "${no_proxy+set}" ]]; then
    echo -e "\e[93mProxy settings are incorrect\e[0m"
    ERR_PROXY=true
    askProxy
  fi
}

checkProxy
