#!/usr/bin/env bash

if  [ $(lsb_release -is) != "Ubuntu" ]; then
  return 0;
fi

function askUfwEnable() {
  echo -e "\e[93mDo you want to enable the ufw firewall?(y/N)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) askUfwRules; return 0;;
      [Nn]* ) echo "Skipped"; return 0;;
      * ) echo "Skipped"; return 0;;
  esac
}

function askUfwDisable() {
  echo -e "\e[93mUfw firewall is active. Allowing traffic on the calico interface."
  sudo ufw allow in on vxlan.calico > /dev/null
  sudo ufw allow out on vxlan.calico > /dev/null
  sudo ufw allow in on cali+ > /dev/null
  sudo ufw allow out on cali+ > /dev/null
  echo -e "\e[93mChoose: [C]ontinue (default), [D]isable ufw, [A]dd rules.\e[0m"
  read -r yn
  case $yn in
      [Dd]* ) sudo ufw disable; return 0;;
      [Cc]* ) echo "Skipped"; return 0;;
      [Aa]* ) askUfwRules; return 0;;
      * ) echo "Skipped"; return 0;;
  esac

}

function askUfwRules() {
  echo -e "\e[93mTo enforce security of the application, you ought to add these rules to ufw firewall:\e[0m"
  echo -e "\e[93m  Incoming default policy: deny\e[0m"
  echo -e "\e[93m  Outgoing default policy: allow\e[0m"
  echo -e "\e[93m  Allow Web ports: 80,443 \n       - From: Anywhere\e[0m"
  echo -e "\e[93m  Allow SSH port: 22 \n       - From networks: 10.0.0.0/8,100.64.0.0/10,172.16.0.0/12,192.168.0.0/16\e[0m"
  echo -e "\e[93m  Allow Grafana, Jaeger Ports: 32222,32121 \n       - From networks: 10.0.0.0/8,100.64.0.0/10,172.16.0.0/12,192.168.0.0/16\e[0m"
  echo -e "\e[93m  Allow k8s ports: 16443,10250,10255,10257,10259 \n       - From networks: 10.0.0.0/8,100.64.0.0/10,172.16.0.0/12,192.168.0.0/16\e[0m"
  echo -e "\e[93mApply recommended rules?(y/N)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) setUfwRules; return 0;;
      [Nn]* ) echo "Skipped"; return 0;;
      * ) echo "Skipped"; return 0;;
  esac
}

function setUfwRules() {
  echo -e "\e[93mWant to clear existing rules??(y/N)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) clearUfwRules;;
      [Nn]* ) echo "Skipped";;
      * ) echo "Skipped";;
  esac

  sudo ufw default deny incoming > /dev/null
  sudo ufw default allow outgoing > /dev/null

  sudo ufw allow in on vxlan.calico > /dev/null
  sudo ufw allow out on vxlan.calico > /dev/null
  sudo ufw allow in on cali+ > /dev/null
  sudo ufw allow out on cali+ > /dev/null

  sudo ufw allow 80 > /dev/null
  sudo ufw allow 443 > /dev/null

  for IP in {10.0.0.0/8,100.64.0.0/10,172.16.0.0/12,192.168.0.0/16}; do
    for PORT in {22,16443,10250,10255,10257,10259,32222,32121}; do
      sudo ufw allow from $IP to any port $PORT > /dev/null
    done;
  done
  echo "Recommended rules added."

  askOwnUfwRules

  sudo ufw --force enable
}

function askOwnUfwRules() {
  echo -e "\e[93mAdd your own allowing rule?(y/N)\e[0m"
  read -r yn
  case $yn in
      [Yy]* ) setOwnUfwRules; return 0;;
      [Nn]* ) echo "Skipped"; return 0;;
      * ) echo "Skipped"; return 0;;
  esac
}

function setOwnUfwRules() {
  echo -n "Enter port (example: 8080): "
  read PORT
  echo -n "Enter IP or network (example: 192.168.1.1 or 192.168.0.0/24): "
  read IP
  sudo ufw allow from $IP to any port $PORT > /dev/null
  echo "Rule 'ufw allow from $IP to any port $PORT' added."
  askOwnUfwRules
}

function clearUfwRules() {
  sudo ufw --force reset > /dev/null
  echo "Firewal has been reset to default."
}

function checkStatusUfw() {
  if ufw status &>/dev/null; then
    if [[ "$(LANG=en_US sudo ufw status)" =~ "inactive" ]]; then
      askUfwEnable
    else
      askUfwDisable
    fi
  else
    return 0;
  fi
}

checkStatusUfw
