#!/usr/bin/env bash

scriptdir="$( dirname "${BASH_SOURCE[0]}" )"
argoyaml=$scriptdir/argocd.yaml

microk8s kubectl create ns argocd > /dev/null 2>&1 || true
microk8s kubectl apply -n argocd -f "${argoyaml}" > /dev/null 2>&1

sleep 1
# Patch for argocd wave healthchecks
microk8s kubectl patch configmap/argocd-cm \
  -n argocd \
  --type merge \
  -p '{"data": {"resource.customizations": "argoproj.io/Application:\n  health.lua: |\n    hs = {}\n    hs.status = \"Healthy\"\n    hs.message = \"\"\n    if obj.status ~= nil then\n      if obj.status.health ~= nil then\n        hs.status = obj.status.health.status\n        hs.message = obj.status.health.message\n      end\n    end\n    return hs\nnetworking.k8s.io/Ingress:\n  health.lua: |\n    hs = {}\n    hs.status = \"Healthy\"\n    return hs\nextensions/Ingress:\n  health.lua: |\n    hs = {}\n    hs.status = \"Healthy\"\n    return hs\n"}}'

if ! microk8s kubectl -n argocd get secret argocd-initial-admin-secret > /dev/null 2>&1 ; then
  pass=$(microk8s kubectl get pods -n argocd -l app.kubernetes.io/name=argocd-server -o name | cut -d'/' -f 2)
  microk8s kubectl -n argocd create secret generic argocd-initial-admin-secret --from-literal=password="${pass}"
fi

sleep 1

cat << EOF | microk8s kubectl apply -n argocd -f -
apiVersion: argoproj.io/v1alpha1
kind: AppProject
metadata:
  name: default
  namespace: argocd
spec:
  clusterResourceWhitelist:
  - group: '*'
    kind: '*'
  destinations:
  - namespace: '*'
    server: '*'
  sourceRepos:
  - '*'
EOF