#!/usr/bin/env bash

if [ "$ELMA365_INSTALL_MODE" != "install" ]; then
  return 0;
fi

function startSurvey() {
  ARGS=""
  if [ -n "$TIMEOUT" ]; then
    ARGS="-t ${TIMEOUT}"
  fi
  if [ "${DEBUG}" == "true" ]; then
    ARGS+=" -D"
  fi
  /usr/local/bin/elma365ctl init -e "$ELMA365_EDITION" --lang "$ELMA365_LANG" $ARGS
}

sleep 5
echo "Waiting for elma365ctl"
while true; do
  microk8s kubectl -n elma365-system wait --for=condition=Ready --timeout=30s pod -l app=elma365ctl > /dev/null 2>&1 && break
done

echo "Waiting for argo controller. It can take a few minutes"
while true; do
  microk8s kubectl -n argocd wait --for=condition=Ready --timeout=30s pod -l app.kubernetes.io/name=argocd-server > /dev/null 2>&1 && break
done

startSurvey

if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'redisUrl' | grep '""' > /dev/null ; then
  REDISLOCAL="true"
else
  REDISLOCAL="false"
fi

if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'mongoUrl' | grep '""' > /dev/null ; then
  MONGOLOCAL="true"
else
  MONGOLOCAL="false"
fi

if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'psqlUrl' | grep '""' > /dev/null ; then
  PSQLLOCAL="true"
else
  PSQLLOCAL="false"
fi

if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'amqpUrl' | grep '""' > /dev/null ; then
  AMQPLOCAL="true"
else
  AMQPLOCAL="false"
fi

if $(microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep s3config -A 1 | tail -1 | sed 's/^.*: //' ); then
  S3LOCAL="false" 
else
  S3LOCAL="true"
fi

cat << EOF | microk8s kubectl apply -n argocd -f -
apiVersion: argoproj.io/v1alpha1
kind: Application
metadata:
  name: elma365-databases
  namespace: argocd
  finalizers:
  - resources-finalizer.argocd.argoproj.io
  annotations:
    argocd.argoproj.io/sync-wave: "10"
spec:
  destination:
    namespace: default
    server: https://kubernetes.default.svc
  project: default
  source:
    path: databases
    repoURL: git://git-server.argocd/elma365.git
    targetRevision: HEAD
    helm:
      valueFiles:
      - values.yaml
      values: |
        postgresql:
          enabled: $PSQLLOCAL
          postgresqlPassword: $PGPASS
        rabbitmq:
          enabled: $AMQPLOCAL
          auth:
            password: $RMQPASS
        mongodb:
          enabled: $MONGOLOCAL
          auth:
            password: $MONGOPASS
            rootPassword: $MONGOROOTPASS
        redis:
          enabled: $REDISLOCAL
        minio:
          enabled: $S3LOCAL
  syncPolicy:
    automated: {}
    syncOptions:
    - CreateNamespace=true
EOF

# microk8s kubectl -n elma365-system create secret generic elma365-dbpass --from-literal=postgresql="${PGPASS}" --from-literal=mongodb="${MONGOPASS}" --from-literal=mongodbroot="${MONGOROOTPASS}" --from-literal=rabbitmq="${RMQPASS}" > /dev/null 2>&1 || true