#!/usr/bin/env bash
set +e

if [ "$ELMA365_INSTALL_MODE" != "upgrade" ] && [ "$ELMA365_INSTALL_MODE" != "upgrade-node" ]; then
  return 0;
fi

CURRENT_MICROK8S_VERSION=$(snap list microk8s 2> /dev/null | tail --lines=+2 | awk '{print $2}')

if [[ "$CURRENT_MICROK8S_VERSION" == v1.19.* ]]; then
  echo "Upgrading k8s to version 1.20"
  snap refresh microk8s --channel=1.20/stable
fi

CURRENT_MICROK8S_VERSION=$(snap list microk8s 2> /dev/null | tail --lines=+2 | awk '{print $2}')

if [[ "$CURRENT_MICROK8S_VERSION" == v1.20.* ]]; then
  echo "Upgrading k8s to version 1.21"
  snap refresh microk8s --channel=1.21/stable
  sleep 30
  microk8s kubectl patch ds/nginx-ingress-microk8s-controller -n ingress --type='json' -p='[{"op": "replace", "path": "/spec/template/spec/containers/0/image", "value": "k8s.gcr.io/ingress-nginx/controller:v0.44.0"}]'
  microk8s kubectl patch ds/nginx-ingress-microk8s-controller -n ingress --type='json' -p='[{"op": "replace", "path": "/spec/template/spec/containers/0/args", "value":["/nginx-ingress-controller","--configmap=$(POD_NAMESPACE)/nginx-load-balancer-microk8s-conf","--tcp-services-configmap=$(POD_NAMESPACE)/nginx-ingress-tcp-microk8s-conf","--udp-services-configmap=$(POD_NAMESPACE)/nginx-ingress-udp-microk8s-conf"," ","--publish-status-address=127.0.0.1"]}]'
  sleep 30
  microk8s kubectl patch role nginx-ingress-microk8s-role -n ingress --type='json' -p='[{"op": "add", "path":"/rules/3", "value":{"apiGroups":[""],"resourceNames":["ingress-controller-leader"],"resources":["configmaps"],"verbs":["create","update"]}}]'
  microk8s kubectl patch role nginx-ingress-microk8s-role -n ingress --type='json' -p='[{"op": "add", "path":"/rules/3", "value":{"apiGroups":[""],"resourceNames":["ingress-controller-leader-nginx"],"resources":["configmaps"],"verbs":["create","update"]}}]'
  sleep 30
  microk8s kubectl patch configmap nginx-load-balancer-microk8s-conf -n ingress --type='merge' -p='{"data":{"annotation-value-word-blocklist":"[\"load_module,lua_package,_by_lua,root,serviceaccount\"]"}}'
#  microk8s kubectl patch deploy coredns -n kube-system --type='json' -p='[{"op": "replace", "path": "/spec/template/spec/containers/0/image", "value":"coredns/coredns:1.8.0"}]'
  sleep 30
  echo "Waiting for running k8s(It can take 10-20 minutes)"

  while true; do
    microk8s kubectl wait --for=condition=Available --timeout=30s deployment --all -n kube-system > /dev/null 2>&1 && break
  done

  microk8s kubectl delete job bootstrap-company -n elma365-system > /dev/null 2>&1
fi

CURRENT_MICROK8S_VERSION=$(snap list microk8s 2> /dev/null | tail --lines=+2 | awk '{print $2}')

if [[ "$CURRENT_MICROK8S_VERSION" == v1.21.* ]]; then
  echo "Patching the ingress k8s version 1.21"
  sleep 15
  microk8s kubectl patch ds/nginx-ingress-microk8s-controller -n ingress --type='json' -p='[{"op": "replace", "path": "/spec/template/spec/containers/0/image", "value": "k8s.gcr.io/ingress-nginx/controller:v0.44.0"}]'
  microk8s kubectl patch role nginx-ingress-microk8s-role -n ingress --type='json' -p='[{"op": "add", "path":"/rules/3", "value":{"apiGroups":[""],"resourceNames":["ingress-controller-leader"],"resources":["configmaps"],"verbs":["create","update"]}}]'
  microk8s kubectl patch role nginx-ingress-microk8s-role -n ingress --type='json' -p='[{"op": "add", "path":"/rules/3", "value":{"apiGroups":[""],"resourceNames":["ingress-controller-leader-nginx"],"resources":["configmaps"],"verbs":["create","update"]}}]'
  microk8s kubectl patch configmap nginx-load-balancer-microk8s-conf -n ingress --type='merge' -p='{"data":{"annotation-value-word-blocklist":"[\"load_module,lua_package,_by_lua,root,serviceaccount\"]"}}'
  microk8s kubectl rollout restart ds/nginx-ingress-microk8s-controller -n ingress
  sleep 15
  echo "Waiting for ingress"
  while true; do
    microk8s kubectl -n ingress wait --for=condition=Ready --timeout=30s pod -l name=nginx-ingress-microk8s > /dev/null 2>&1 && break
  done
fi

for ARG in /var/snap/microk8s/current/args/kube-scheduler /var/snap/microk8s/current/args/kube-controller-manager; do
  if ! grep -iqe "leader-elect-lease-duration" -iqe "leader-elect-renew-deadline" $ARG; then
    MICROK8S_RESTART=true
    echo -e "--leader-elect-lease-duration=60s\n--leader-elect-renew-deadline=30s" | sudo tee -a $ARG > /dev/null
  fi
done

if [[ ${MICROK8S_RESTART} == true ]]; then
  microk8s stop && microk8s start
  microk8s status --wait-ready > /dev/null
  sleep 30
fi

snap set system snapshots.automatic.retention=no

if [ "$ELMA365_INSTALL_MODE" == "upgrade-node" ]; then
  return 0;
fi

echo "Waiting for argo controller"
while true; do
  microk8s kubectl -n argocd wait --for=condition=Ready --timeout=30s pod -l app.kubernetes.io/name=argocd-server > /dev/null 2>&1 && break
done

echo "Waiting for git-server ver. "${ELMA365_SETUP_VERSION}
while true; do
  microk8s kubectl -n argocd wait --for=condition=Ready --timeout=30s pod -l app.kubernetes.io/instance=git-server-${ELMA365_SETUP_VERSION} > /dev/null 2>&1 && break
  sleep 5
done

echo "Waiting for elma365ctl ver. "${ELMA365_SETUP_VERSION}
while true; do
  microk8s kubectl -n elma365-system wait --for=condition=Ready --timeout=30s pod -l app.kubernetes.io/instance=elma365ctl-${ELMA365_SETUP_VERSION} > /dev/null 2>&1 && break
  sleep 15
done

while true; do
  version=$(elma365ctl version 2>/dev/null) || true
  if [ "$version" != "$ELMA365_SETUP_VERSION" ]; then
    sleep 5
  else
    break
  fi
done

echo "Waiting for rabbitmq version check"
while true; do
  if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'amqpUrl' | grep '""' > /dev/null; then
    break
  fi
  amqpVersWork="3.9.0"
  amqpVers=$(elma365ctl checkamqpversion 2>/dev/null) || true
  if [ -z "$amqpVers" ]; then
    break
  fi
  semver_result=$(./../semver.sh compare $amqpVersWork $amqpVers)
  [ "$semver_result" == "-1" ] && { break; }
  if [ "$semver_result" == "1" ] || [ "$amqpVers" == "unknown" ]; then
    echo -e "\e[93mFor the correct operation of the ELMA365 application, make sure that the version of RabbitMQ is 3.9.9 or later.\e[0m"
    echo -e "\e[93m"More details in the help https://elma365.com/ru/help/rabbitmq.html"\e[0m"
    echo -e "\e[93mPress [ENTER] to continue...\e[0m"
    read -r yn
    case $yn in
        * ) break;;
    esac
    sleep 5
  else
    break
  fi
done

echo "Waiting for rabbitmq plugin check"
while true; do
  if microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep 'amqpUrl' | grep '""' > /dev/null; then
    break
  fi
  amqpPlg=$(elma365ctl checkamqpplugin 2>/dev/null) || true
  if [ "$amqpPlg" == "disable" ] || [ "$amqpPlg" == "unknown" ]; then
    echo -e "\e[93mFor external RabbitMQ, make sure plugin rabbitmq_consistent_hash_exchange enabled.\e[0m"
    echo -e "\e[93m"More details in the help https://elma365.com/ru/help/rabbitmq.html"\e[0m"
    echo -e "\e[93mPress [ENTER] to continue...\e[0m"
    read -r yn
    case $yn in
        * ) break;;
    esac
    sleep 5
  else
    break
  fi
done

echo "Start upgrade services"
while true; do
  version=$(elma365ctl upgrade 2>/dev/null) || true
  if [ "$version" != "$ELMA365_SETUP_VERSION" ]; then
    sleep 30
  else
    break
  fi
done
