#!/usr/bin/env bash

#sleep 5
#echo "Waiting for elma365ctl"
#while true; do
#  microk8s kubectl -n elma365-system wait --for=condition=Ready --timeout=1s pod -l app=elma365ctl > /dev/null 2>&1 && break
#done


if [ "$ELMA365_INSTALL_MODE" != "upgrade" ]; then
  return 0;
fi

echo "Waiting for ELMA365 application upgrade"
sleep 60
while true; do
  status=$(microk8s kubectl -n argocd get Application elma365 -o jsonpath='{.status.health.status}' 2>/dev/null) || true
  syncstatus=$(microk8s kubectl -n argocd get Application elma365 -o jsonpath='{.status.sync.status}' 2>/dev/null) || true
  phase=$(microk8s kubectl -n argocd get Application elma365 -o jsonpath='{.status.operationState.phase}' 2>/dev/null) || true

  if [ "$phase" != "Succeeded" ] || [ "$status" != "Healthy" ] || [ "$syncstatus" != "Synced" ]; then
    microk8s kubectl -n argocd get Application elma365 -o jsonpath='{.status.operationState.message}' 2>/dev/null && echo || true
    sleep 15
  else
    break
  fi
done

echo "Upgrade ELMA365 application to version ${ELMA365_SETUP_VERSION} successful"

if [ "$ELMA365_EDITION" = "enterprise" ]; then
  if ! microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep -A 3 'autoscaling:' | grep 'enabled: true' > /dev/null \
  || ! microk8s kubectl get cm elma365-survey -n elma365-system -o yaml | grep -A 1 'linkerd:' | grep 'enabled: true' > /dev/null; then
    read -p $'\e[93mSetup Autoscaling? (System requirements in the help https://elma365.com/ru/help/elma365-enterprise-on-premises.html)[Y/n]\e[0m' -n 1 -r
    echo    # (optional) move to a new line
    if [[ $REPLY =~ ^[Nn]$ ]]; then
      echo "Skipped"
    else
      echo "Scaling services into one replica"
      elma365ctl scale all 1
      elma365ctl postupgrade
      echo "Reconfiguration started. Please wait, it will be completed in about 5-10 minutes"
    fi
  fi
fi

